from django.db import models


class LRNDKey(models.Model):
    key = models.TextField()
    created_at = models.DateTimeField(auto_now_add=True, use_tz=False)
    updated_at = models.DateTimeField(auto_now=True, use_tz=False)

    def __str__(self):
        return f"LRND Key {self.id}"


import json

from django.apps import apps
from django.conf import settings
from django.contrib import messages
from django.http import JsonResponse
from django.shortcuts import redirect, render
from django.utils.decorators import method_decorator
from django.views import View
from django.views.decorators.csrf import csrf_exempt

from .middleware import LRNDMiddleware


@method_decorator(csrf_exempt, name='dispatch')
class ValidateView(View):
    template_name = 'djangoLrnd/validate.html'

    def get(self, request):
        return render(request, self.template_name)

    def post(self, request):
        try:
            key = request.POST.get('key')
            
            if not key:
                messages.error(request, 'Kunci tidak ditemukan dalam data')
                return redirect('lrnd_validate')
            
            response = LRNDMiddleware.check_key_status(key)
            
            if response.status_code == 200:
                response_data = response.json()
                if response_data.get('error') == 'Invalid Key':
                    messages.error(request, 'Kunci tidak valid')
                    return redirect('lrnd_validate')
                elif response_data.get('error') == 'Used':
                    messages.error(request, 'Kunci sudah digunakan')
                    return redirect('lrnd_validate')
                elif response_data.get('error') == 'Invalid Key Name':
                    messages.error(request, 'Nama kunci tidak valid')
                    return redirect('lrnd_validate')
                elif response_data.get('message') == 'Successfully':
                    LRNDKey = apps.get_model('djangoLrnd', 'LRNDKey')
                    LRNDKey.objects.update_or_create(id=1, defaults={'key': key})
                    success_redirect_url = getattr(settings, 'LRND_SUCCESS_REDIRECT_URL', '')
                    messages.success(request, 'Kunci berhasil divalidasi')
                    return redirect(success_redirect_url)
                else:
                    messages.error(request, 'Terjadi kesalahan saat memvalidasi kunci')
                    return redirect('lrnd_validate')
            else:
                messages.error(request, 'Terjadi kesalahan saat memvalidasi kunci')
                return redirect('lrnd_validate')
        
        except json.JSONDecodeError:
            messages.error(request, 'Data JSON tidak valid')
            return redirect('lrnd_validate')
        except Exception as e:
            messages.error(request, str(e))
            return redirect('lrnd_validate')

validate_view = ValidateView.as_view()


import base64
import json
from datetime import datetime

import requests
from cryptography.fernet import Fernet
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from django.apps import apps
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.shortcuts import redirect
from django.urls import reverse


class LRNDMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response
        self.validation_endpoint = getattr(settings, 'LRND_VALIDATION_ENDPOINT', 'http://127.0.0.1:8000/validate/')
        self.encryption_password = getattr(settings, 'LRND_ENCRYPTION_PASSWORD', 'password')
        self.exempt_paths = getattr(settings, 'LRND_EXEMPT_PATHS', ['/validate/'])
        self.key_model = getattr(settings, 'LRND_KEY_MODEL', 'djangoLrnd.LRNDKey')

    def __call__(self, request):
        if not self.is_path_exempt(request.path):
            if not self.is_valid(request):
                return redirect(reverse('lrnd_validate'))
        response = self.get_response(request)
        return response

    def is_path_exempt(self, path):
        return any(path.startswith(exempt_path) for exempt_path in self.exempt_paths)

    def is_valid(self, request):
        try:
            LRNDKey = apps.get_model('djangoLrnd', 'LRNDKey')
            key_instance = LRNDKey.objects.first()
        except Exception:
            raise ImproperlyConfigured("Tidak dapat mengambil instance LRNDKey")

        if not key_instance:
            return False

        try:
            decrypted_data = self.decrypt_data(key_instance.key, self.encryption_password)
            expiry_date = json.loads(decrypted_data)['expiry_date']
            expiry_date = datetime.fromisoformat(expiry_date)
            return expiry_date > datetime.now().astimezone()
        except Exception:
            return False

    @staticmethod
    def check_key_status(key):
        password = getattr(settings, 'LRND_ENCRYPTION_PASSWORD', 'password')
        response = requests.post(settings.LRND_VALIDATION_ENDPOINT, json={'key': key, 'password': password}, headers={'Content-Type': 'application/json'})
        return response

    @staticmethod
    def create_key(password: str, salt: bytes) -> bytes:
        kdf = PBKDF2HMAC(
            algorithm=hashes.SHA256(),
            length=32,
            salt=salt,
            iterations=100000,
            backend=default_backend()
        )
        return base64.urlsafe_b64encode(kdf.derive(password.encode()))

    @classmethod
    def decrypt_data(cls, encrypted_data: str, password: str) -> str:
        decoded = base64.urlsafe_b64decode(encrypted_data)
        salt = decoded[:16]
        encrypted = decoded[16:]
        key = cls.create_key(password, salt)
        fernet = Fernet(key)
        return fernet.decrypt(encrypted).decode()